#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, with_statement, unicode_literals

import os
import os.path
import unittest
import datetime
import shutil

from simiki.config import parse_config, get_default_config
from simiki.utils import copytree, emptytree
from simiki.generators import PageGenerator
from simiki.compat import unicode

test_path = os.path.dirname(os.path.abspath(__file__))
base_path = os.path.dirname(test_path)


class TestPageGenerator(unittest.TestCase):
    def setUp(self):
        self.wiki_path = os.path.join(test_path, 'mywiki_for_generator')

        os.chdir(self.wiki_path)

        self.config_file = os.path.join(base_path, 'simiki',
                                        'conf_templates', '_config.yml.in')

        self.config = parse_config(self.config_file)

        s_themes_path = os.path.join(base_path, 'simiki', 'themes')
        self.d_themes_path = os.path.join('./', 'themes')
        if os.path.exists(self.d_themes_path):
            shutil.rmtree(self.d_themes_path)
        copytree(s_themes_path, self.d_themes_path)

    def test_get_category_and_file(self):
        src_file_path = os.path.join(self.wiki_path, 'content', 'foo目录',
                                     'foo_page_中文.md')
        generator = PageGenerator(self.config, self.wiki_path, src_file_path)
        category, filename = generator.get_category_and_file()
        self.assertEqual(
            (category, filename),
            (u'foo\u76ee\u5f55', u'foo_page_\u4e2d\u6587.md')
        )

    def test_get_meta_and_content(self):
        src_file_path = os.path.join(self.wiki_path, 'content', 'foo目录',
                                     'foo_page_中文.md')
        generator = PageGenerator(self.config, self.wiki_path, src_file_path)
        meta, content = generator.get_meta_and_content()
        expected_meta = {'date': '2013-10-17 00:03', 'layout': 'page',
                         'title': 'Foo Page 2'}
        self.assertEqual(meta, expected_meta)
        self.assertEqual(content, '<p>Simiki is a simple wiki '
                                  'framework, written in Python.</p>')

        # get meta notaion error
        src_file_path = os.path.join(self.wiki_path, 'content', 'foo目录',
                                     'foo_page_中文_meta_error_1.md')
        generator = PageGenerator(self.config, self.wiki_path,
                                    src_file_path)
        self.assertRaises(Exception, generator.get_meta_and_content)

        src_file_path = os.path.join(self.wiki_path, 'content', 'foo目录',
                                       'foo_page_中文_meta_error_2.md')
        generator = PageGenerator(self.config, self.wiki_path,
                                    src_file_path)
        self.assertRaises(Exception, generator.get_meta_and_content)

    def test_get_template_vars(self):
        src_file_path = os.path.join(self.wiki_path, 'content', 'foo目录',
                                     'foo_page_中文.md')
        generator = PageGenerator(self.config, self.wiki_path, src_file_path)
        meta, content = generator.get_meta_and_content()
        template_vars = generator.get_template_vars(meta, content)
        expected_template_vars = {
            u'page': {
                u'category': u'foo\u76ee\u5f55',
                u'content': u'<p>Simiki is a simple wiki '
                            'framework, written in Python.</p>',
                u'filename': u'foo_page_\u4e2d\u6587.html',
                u'date': '2013-10-17 00:03',
                u'layout': 'page',
                u'title': 'Foo Page 2'
            },
            u'site': get_default_config()
        }

        expected_template_vars['site'].update({'root': ''})
        template_vars['site'].pop('time')
        expected_template_vars['site'].pop('time')
        assert template_vars == expected_template_vars

    def test_to_html(self):
        src_file_path = os.path.join(self.wiki_path, 'content', 'foo目录',
                                     'foo_page_中文.md')
        generator = PageGenerator(self.config, self.wiki_path, src_file_path)
        html = generator.to_html().strip()
        expected_output = os.path.join(self.wiki_path, 'expected_output.html')
        fd = open(expected_output, "rb")
        year = datetime.date.today().year
        expected_html = unicode(fd.read(), "utf-8") % year
        assert html == expected_html

        # load template error
        src_file_path = os.path.join(self.wiki_path, 'content', 'foo目录',
                                     'foo_page_中文.md')
        self.assertRaises(Exception, PageGenerator, self.config,
                          'wrong_basepath', src_file_path)

    def test_get_layout(self):
        src_file_path = os.path.join(self.wiki_path, 'content', 'foo目录',
                                     'foo_page_layout_old_post.md')
        generator = PageGenerator(self.config, self.wiki_path, src_file_path)
        meta, _ = generator.get_meta_and_content()

        layout = generator.get_layout(meta)
        self.assertEqual(layout, 'page')

        src_file_path = os.path.join(self.wiki_path, 'content', 'foo目录',
                                     'foo_page_layout_without_layout.md')
        generator = PageGenerator(self.config, self.wiki_path, src_file_path)
        meta, _ = generator.get_meta_and_content()

        layout = generator.get_layout(meta)
        self.assertEqual(layout, 'page')

    def test_get_meta(self):
        src_file_path = os.path.join(self.wiki_path, 'content', 'foo目录',
                                     'foo_page_get_meta_yaml_error.md')
        generator = PageGenerator(self.config, self.wiki_path, src_file_path)
        self.assertRaises(Exception, generator.get_meta_and_content)

        src_file_path = os.path.join(self.wiki_path, 'content', 'foo目录',
                                     'foo_page_get_meta_without_title.md')
        generator = PageGenerator(self.config, self.wiki_path, src_file_path)
        self.assertRaises(Exception, generator.get_meta_and_content)

    def tearDown(self):
        if os.path.exists(self.d_themes_path):
            shutil.rmtree(self.d_themes_path)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals, absolute_import

import os
import os.path
import shutil
import logging

from simiki.config import parse_config
from simiki.utils import (copytree, mkdir_p, listdir_nohidden)


class Initiator(object):
    conf_template_dn = "conf_templates"
    config_fn = "_config.yml"
    fabfile_fn = "fabfile.py"
    demo_fn = "gettingstarted.md"

    def __init__(self, config_file, target_path):
        self.config_file = config_file
        self.config = parse_config(self.config_file)
        self.source_path = os.path.dirname(__file__)
        self.target_path = target_path

    @staticmethod
    def get_file(src, dst):
        if os.path.exists(dst):
            logging.warning("{0} exists".format(dst))
            return

        # Create parent directory
        dst_directory = os.path.dirname(dst)
        if not os.path.exists(dst_directory):
            mkdir_p(dst_directory)
            logging.info("Creating directory: {0}".format(dst_directory))

        shutil.copyfile(src, dst)
        logging.info("Creating file: {0}".format(dst))

    def get_config_file(self):
        dst_config_file = os.path.join(self.target_path, self.config_fn)
        self.get_file(self.config_file, dst_config_file)

    def get_fabfile(self):
        src_fabfile = os.path.join(
            self.source_path,
            self.conf_template_dn,
            self.fabfile_fn
        )
        dst_fabfile = os.path.join(self.target_path, self.fabfile_fn)
        self.get_file(src_fabfile, dst_fabfile)

    def get_demo_page(self):
        nohidden_dir = listdir_nohidden(
            os.path.join(self.target_path, self.config['source']))
        # If there is file/directory under content, do not create first page
        if next(nohidden_dir, False):
            return

        src_demo = os.path.join(self.source_path, self.conf_template_dn,
                                self.demo_fn)
        dst_demo = os.path.join(self.target_path, "content", "intro",
                                self.demo_fn)
        self.get_file(src_demo, dst_demo)

    def get_default_theme(self, theme_path):
        default_theme_name = self.config['theme']
        src_theme = os.path.join(self.source_path, self.config['themes_dir'],
                                 default_theme_name)
        dst_theme = os.path.join(theme_path, default_theme_name)
        if os.path.exists(dst_theme):
            logging.warning('{0} exists'.format(dst_theme))
        else:
            copytree(src_theme, dst_theme)
            logging.info("Copying default theme '{0}' to: {1}"
                         .format(default_theme_name, theme_path))

    def init(self):
        content_path = os.path.join(self.target_path, self.config["source"])
        output_path = os.path.join(self.target_path,
                                   self.config["destination"])
        theme_path = os.path.join(self.target_path, self.config['themes_dir'])
        for path in (content_path, output_path, theme_path):
            if os.path.exists(path):
                logging.warning("{0} exists".format(path))
            else:
                mkdir_p(path)
                logging.info("Creating directory: {0}".format(path))

        self.get_config_file()
        self.get_fabfile()
        self.get_demo_page()
        self.get_default_theme(theme_path)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Jinja2 custom filters and extensions
'''
import datetime
import tzlocal
from simiki.compat import basestring

filters = ['rfc3339']


def rfc3339(dt_obj):
    '''
    dt_obj: datetime object or string

    The filter use `datetime.datetime.isoformat()`, which is in ISO 8601
    format, not in RFC 3339 format, but they have a lot in common, so I used
    ISO 8601 format directly.
    '''
    if isinstance(dt_obj, datetime.datetime):
        pass
    elif isinstance(dt_obj, basestring):
        for fmt in ('%Y-%m-%d %H:%M:%S', '%Y-%m-%d %H:%M'):
            try:
                dt_obj = datetime.datetime.strptime(dt_obj, fmt)
            except ValueError:
                pass
            else:
                break
        else:
            raise ValueError('can not parse datetime {0}'.format(dt_obj))
    else:
        raise ValueError('{0} is not datetime object or string'.format(dt_obj))
    # make sure the dt_obj is local time
    if not dt_obj.tzinfo:
        tz = tzlocal.get_localzone()
        dt_obj = tz.localize(dt_obj)
    # remove microsecond
    dt_obj = dt_obj.replace(microsecond=0)
    return dt_obj.isoformat()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, with_statement, unicode_literals
import unittest
import logging

try:
    # py2
    from cStringIO import StringIO
except ImportError:
    try:
        # py2
        from StringIO import StringIO
    except ImportError:
        # py3
        # from io import BytesIO as StringIO
        from io import StringIO as StringIO

import nose

from simiki.utils import color_msg
from simiki.log import logging_init
from simiki.compat import is_py2


class TestLogInit(unittest.TestCase):

    def setUp(self):
        logging.disable(logging.NOTSET)
        self.stream = StringIO()
        self.logger = logging.getLogger()
        self.handler = logging.StreamHandler(self.stream)
        for handler in self.logger.handlers:
            # exclude nosetest capture handler
            if not isinstance(handler,
                              nose.plugins.logcapture.MyMemoryHandler):
                self.logger.removeHandler(handler)
        logging_init(level=logging.DEBUG, handler=self.handler)

    def test_logging_init(self):
        l2c = {
            "debug": "blue",
            "info": "green",
            "warning": "yellow",
            "error": "red",
            "critical": "bgred"
        }
        for level in l2c:
            # self.handler.flush()
            self.stream.truncate(0)
            # in python 3.x, truncate(0) would not change the current file pos
            # via <http://stackoverflow.com/a/4330829/1276501>
            self.stream.seek(0)
            func = getattr(self.logger, level)
            func(level)
            expected_output = "[{0}]: {1}" \
                .format(color_msg(l2c[level], level.upper()), level)
            stream_output = self.stream.getvalue().strip()
            if is_py2:
                stream_output = unicode(stream_output)
            self.assertEqual(stream_output, expected_output)

    def tearDown(self):
        logging.disable(logging.CRITICAL)
        for handler in self.logger.handlers:
            if not isinstance(handler,
                              nose.plugins.logcapture.MyMemoryHandler):
                self.logger.removeHandler(handler)

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" Convert Markdown file to html, which is embeded in html template.
"""

from __future__ import (print_function, with_statement, unicode_literals,
                        absolute_import)

import os
import os.path
import io
import copy
import traceback
try:
    from collections import OrderedDict
except ImportError:
    from ordereddict import OrderedDict

import markdown
import yaml
from jinja2 import (Environment, FileSystemLoader, TemplateError)

from simiki import jinja_exts
from simiki.compat import is_py2, is_py3

if is_py3:
    from functools import cmp_to_key

PLAT_LINE_SEP = '\n'


class BaseGenerator(object):
    """Base generator class"""

    def __init__(self, site_config, base_path):
        '''
        :site_config: site global configuration parsed from _config.yml
        :base_path: root path of wiki directory
        '''
        self.site_config = copy.deepcopy(site_config)
        self.base_path = base_path
        _template_path = os.path.join(
            self.base_path,
            site_config["themes_dir"],
            site_config["theme"]
        )
        if not os.path.exists(_template_path):
            raise Exception("Theme `{0}' not exists".format(_template_path))
        self.env = Environment(
            loader=FileSystemLoader(_template_path)
        )
        self._jinja_load_exts()

    def _jinja_load_exts(self):
        '''Load jinja custom filters and extensions'''
        for _filter in jinja_exts.filters:
            self.env.filters[_filter] = getattr(jinja_exts, _filter)


class PageGenerator(BaseGenerator):

    def __init__(self, site_config, base_path, src_file_path):
        '''
        :src_file_path: path of a source file
        '''
        super(PageGenerator, self).__init__(site_config, base_path)
        self.src_file_path = src_file_path
        # source file path relative to base_path
        self.src_file_relpath = os.path.relpath(src_file_path, self.base_path)
        self.meta = None
        self.content = None

    def to_html(self):
        """Load template, and generate html"""
        self.meta, self.content = self.get_meta_and_content()
        if self.meta.get('draft', False):
            return None
        layout = self.get_layout(self.meta)
        template_file = "{0}.html".format(layout)
        template_vars = self.get_template_vars(self.meta, self.content)
        try:
            template = self.env.get_template(template_file)
            html = template.render(template_vars)
        except TemplateError:
            # jinja2.exceptions.TemplateNotFound will get blocked
            # in multiprocessing?
            exc_msg = "unable to load template '{0}'\n{1}" \
                      .format(template_file, traceback.format_exc())
            raise Exception(exc_msg)

        return html

    def get_meta_and_content(self):
        """Split the source file texts by triple-dashed lines, return the mata
        and content.
        meta is page's meta data, dict type.
        content is html parsed from markdown or other markup text.
        """
        meta_notation = "---"
        with io.open(self.src_file_path, "rt", encoding="utf-8") as fd:
            textlist = fd.read().lstrip().splitlines()
            if textlist[0] != meta_notation:
                raise Exception("disallow anything except newline "
                                "before begin meta notation '---'")
            textlist = textlist[1:]

        try:
            second_meta_notation_index = textlist.index(meta_notation)
        except ValueError:
            raise Exception("can't find end meta notation '---'")
        meta_textlist = textlist[:second_meta_notation_index]
        markup_textlist = textlist[second_meta_notation_index+1:]

        meta = self._get_meta(PLAT_LINE_SEP.join(meta_textlist))
        markup_text = PLAT_LINE_SEP.join(markup_textlist)
        if meta.get('render', True):
            content = self._parse_markdown(markup_text)
        else:
            content = markup_text

        return (meta, content)

    @staticmethod
    def get_layout(meta):
        """Get layout config in meta, default is 'page'"""
        if "layout" in meta:
            # Compatible with previous version, which default layout is "post"
            # XXX Will remove this checker in v2.0
            if meta["layout"] == "post":
                layout = "page"
            else:
                layout = meta["layout"]
        else:
            layout = "page"

        return layout

    def get_template_vars(self, meta, content):
        """Get template variables, include site config and page config"""
        category, src_fname = self.get_category_and_file()
        dst_fname = src_fname.replace(
            ".{0}".format(self.site_config["default_ext"]), ".html")
        page = {
            "category": category,
            "content": content,
            "filename": dst_fname
        }
        page.update(meta)
        template_vars = {
            "site": self.site_config,
            "page": page,
        }

        # if site.root endswith `/`, remove it.
        site_root = template_vars["site"]["root"]
        if site_root.endswith("/"):
            template_vars["site"]["root"] = site_root[:-1]

        return template_vars

    def get_category_and_file(self):
        """Get the name of category and file(with extension)"""
        src_file_relpath_to_source = \
            os.path.relpath(self.src_file_relpath, self.site_config['source'])
        category, filename = os.path.split(src_file_relpath_to_source)
        return (category, filename)

    def _get_meta(self, meta_yaml):
        """Get meta and validate them

        :param meta_yaml: meta in yaml format
        """
        try:
            meta = yaml.load(meta_yaml)
        except yaml.YAMLError as e:
            e.extra_msg = 'yaml format error'
            raise

        if not self._check_meta(meta):
            raise Exception("no 'title' in meta")

        return meta

    def _check_meta(self, meta):
        """Check if meta is right"""
        is_meta_right = True
        if "title" not in meta:
            is_meta_right = False
        return is_meta_right

    def _parse_markdown(self, markdown_content):
        """Parse markdown text to html"""
        markdown_extensions = self._set_markdown_extensions()

        html_content = markdown.markdown(
            markdown_content,
            extensions=markdown_extensions,
        )

        return html_content

    def _set_markdown_extensions(self):
        """Set the extensions for markdown parser"""
        # TODO: custom markdown extension in _config.yml
        # Base markdown extensions support "fenced_code".
        markdown_extensions = ["fenced_code"]
        if self.site_config["pygments"]:
            markdown_extensions.extend([
                "extra",
                "codehilite(css_class=hlcode)",
                "toc(title=Table of Contents)"
            ])

        return markdown_extensions


class CatalogGenerator(BaseGenerator):

    def __init__(self, site_config, base_path, pages):
        '''
        :pages: all pages' meta variables, dict type
        '''
        super(CatalogGenerator, self).__init__(site_config, base_path)
        self.pages = pages

    def get_content_structure_and_meta(self):
        """Ref: http://stackoverflow.com/a/9619101/1276501"""
        dct = {}
        ext = self.site_config["default_ext"]
        for path, meta in self.pages.items():
            # Ignore other files
            if not path.endswith(ext):
                continue
            p = dct
            for x in path.split(os.sep):
                if ext in x:
                    meta["name"] = os.path.splitext(x)[0]
                    p = p.setdefault(x, meta)
                else:
                    p = p.setdefault(x, {})

        return dct.get(self.site_config['source'], {})

    def sort_structure(self, structure):
        """Sort index structure in lower-case, alphabetical order

        Compare argument is a key/value structure, if the compare argument is a
        leaf node, which has `title` key in its value, use the title value,
        else use the key to compare.
        """

        def _cmp(arg1, arg2):
            arg1 = arg1[1]["title"] if "title" in arg1[1] else arg1[0]
            arg2 = arg2[1]["title"] if "title" in arg2[1] else arg2[0]
            # cmp not exists in py3
            # via <https://docs.python.org/3.0/whatsnew/3.0.html#ordering-comparisons>
            cmp = lambda x, y: (x > y) - (x < y)
            return cmp(arg1.lower(), arg2.lower())

        if is_py2:
            sorted_opts = {'cmp': _cmp}
        elif is_py3:
            sorted_opts = {'key': cmp_to_key(_cmp)}

        sorted_structure = copy.deepcopy(structure)
        for k, _ in sorted_structure.items():
            sorted_structure = OrderedDict(sorted(
                sorted_structure.items(),
                **sorted_opts
            ))
            if k.endswith(".{0}".format(self.site_config["default_ext"])):
                continue
            sorted_structure[k] = self.sort_structure(sorted_structure[k])
        return sorted_structure

    def get_template_vars(self):
        self.site_config["structure"] = \
            self.sort_structure(self.get_content_structure_and_meta())
        tpl_vars = {
            "site": self.site_config,
        }

        # if site.root endwith `\`, remote it.
        site_root = tpl_vars["site"]["root"]
        if site_root.endswith("/"):
            tpl_vars["site"]["root"] = site_root[:-1]

        return tpl_vars

    def generate_catalog_html(self):
        tpl_vars = self.get_template_vars()
        html = self.env.get_template("index.html").render(tpl_vars)
        return html


class FeedGenerator(BaseGenerator):
    def __init__(self, site_config, base_path, pages, feed_fn='atom.xml'):
        '''
        :pages: all pages' meta variables, dict type
        '''
        super(FeedGenerator, self).__init__(site_config, base_path)
        self.pages = pages
        self.feed_fn = feed_fn

    def get_template_vars(self):
        tpl_vars = {
            "site": self.site_config,
            "pages": self.pages
        }

        # if site.root endwith `\`, remote it.
        site_root = tpl_vars["site"]["root"]
        if site_root.endswith("/"):
            tpl_vars["site"]["root"] = site_root[:-1]

        return tpl_vars

    def generate_feed(self):
        tpl_vars = self.get_template_vars()
        with open(os.path.join(self.base_path, self.feed_fn), 'r') as fd:
            template = self.env.from_string(fd.read())
            feed_content = template.render(tpl_vars)
        return feed_content



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals
import os
import os.path
import unittest
from simiki import utils

test_path = os.path.dirname(os.path.abspath(__file__))

class TestUtils(unittest.TestCase):

    def setUp(self):
        wiki_path = os.path.join(test_path, 'mywiki_for_others')
        os.chdir(wiki_path)
        self.content = 'content'
        self.output = 'output'
        if os.path.exists(self.output):
            utils.emptytree(self.output)
            os.rmdir(self.output)

    def test_check_extension(self):
        valid_files = ['/tmp/file1.md', '/tmp/file2.mkd', '/tmp/文件3.mdown',
                       '/tmp/文件4.markdown', '/tmp/目录/文件5.md']
        for f in valid_files:
            assert utils.check_extension(f)

        invalid_files = ['/tmp/file6.txt', '/tmp/目录/文件7.mkdown']
        for f in invalid_files:
            assert not utils.check_extension(f)

    def test_copytree_common(self):
        utils.copytree(self.content, self.output)
        assert os.path.exists(self.output) and os.path.isdir(self.output)

        files = [
            os.path.join('python', 'zen_of_python.md'),
            os.path.join('python', 'python文档.md'),
            os.path.join('其它', 'helloworld.markdown'),
            os.path.join('其它', '维基.md'),
            os.path.join('其它', 'hello.txt'),
            os.path.join('其它', '.hidden.md'),
        ]
        for f in files:
            assert os.path.exists(os.path.join(self.output, f))

    def test_copytree_symlink(self):
        '''temp not support'''
        pass

    def test_copytree_ignore(self):
        '''temp not support'''
        pass

    def test_emptytree(self):
        utils.copytree(self.content, self.output)
        utils.emptytree(self.output)
        assert not os.listdir(self.output)

    def test_mkdir_p(self):
        path = os.path.join(self.content)
        utils.mkdir_p(path)
        assert os.path.exists(path)

        path = os.path.join(self.output, "dir1/dir2/dir3")
        utils.mkdir_p(path)
        assert os.path.exists(path)

        # test path exist, and not a directory
        path = os.path.join(self.content, '其它', 'hello.txt')
        self.assertRaises(OSError, lambda: utils.mkdir_p(path))

    def test_listdir_nohidden(self):
        fs = utils.listdir_nohidden(os.path.join(self.content, '其它'))
        expected_listdir = ['hello.txt', 'helloworld.markdown', '维基.md']
        assert sorted(list(fs)) == sorted(expected_listdir)

    def tearDown(self):
        if os.path.exists(self.output):
            utils.emptytree(self.output)
            os.rmdir(self.output)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import

import os
import logging
import time
from watchdog.observers import Observer
from watchdog.events import PatternMatchingEventHandler
import simiki
from simiki.generators import PageGenerator
from simiki.utils import write_file

_site_config = None
_base_path = None


class YAPatternMatchingEventHandler(PatternMatchingEventHandler):
    '''Observe .md files under content directory.
    Temporary only regenerate, not delete unused files'''
    patterns = ['*.{0}'.format(e) for e in simiki.allowed_extensions]

    @staticmethod
    def generate_page(_file):
        pg = PageGenerator(_site_config, _base_path, _file)
        html = pg.to_html()
        category, filename = os.path.split(_file)
        category = os.path.relpath(category, _site_config['source'])
        output_fname = os.path.join(
            _base_path,
            _site_config['destination'],
            category,
            '{0}.html'.format(os.path.splitext(filename)[0])
        )
        write_file(output_fname, html)
        logging.debug('Regenerating: {0}'.format(_file))

    def process(self, event):
        if event.event_type in ('moved',):
            _file = event.dest_path
        else:
            _file = event.src_path
        self.generate_page(_file)

    def on_created(self, event):
        self.process(event)

    def on_modified(self, event):
        self.process(event)

    def on_moved(self, event):
        self.process(event)


def watch(site_config, base_path):
    global _site_config, _base_path
    _site_config = site_config
    _base_path = base_path

    observe_path = os.path.join(_base_path, _site_config['source'])
    event_handler = YAPatternMatchingEventHandler()
    observer = Observer()
    observer.schedule(event_handler, observe_path, recursive=True)
    observer.start()
    try:
        while True:
            time.sleep(1)
    except (KeyboardInterrupt, SystemExit):
        logging.info("Shutting down watcher")
        observer.stop()
    observer.join()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Simiki CLI

Usage:
  simiki init [-p <path>]
  simiki new | n -t <title> -c <category> [-f <file>]
  simiki generate | g
  simiki preview | p [--host <host>] [--port <port>] [-w]
  simiki update
  simiki -h | --help
  simiki -V | --version

Subcommands:
  init                Initial site
  new                 Create a new wiki page
  generate            Generate site
  preview             Preview site locally (develop mode)
  update              Update builtin scripts and themes under local site

Options:
  -h, --help          Help information.
  -V, --version       Show version.
  -p <path>           Specify the target path.
  -c <category>       Specify the category.
  -t <title>          Specify the new post title.
  -f <file>           Specify the new post filename.
  --host <host>       Bind host to preview [default: localhost]
  --port <port>       Bind port to preview [default: 8000]
  -w                  Auto regenerated when file changed
"""

from __future__ import print_function, unicode_literals, absolute_import

import os
import os.path
import sys
import io
import datetime
import shutil
import logging
import random
import multiprocessing
import time
import hashlib

from docopt import docopt
from yaml import YAMLError

from simiki.generators import (PageGenerator, CatalogGenerator, FeedGenerator)
from simiki.initiator import Initiator
from simiki.config import parse_config
from simiki.log import logging_init
from simiki.server import preview
from simiki.watcher import watch
from simiki.utils import (copytree, emptytree, mkdir_p, write_file)
from simiki import __version__
from simiki.compat import unicode, basestring, xrange, raw_input

try:
    from os import getcwdu
except ImportError:
    from os import getcwd as getcwdu

logger = logging.getLogger(__name__)
config = None


def init_site(target_path):
    default_config_file = os.path.join(os.path.dirname(__file__),
                                       "conf_templates",
                                       "_config.yml.in")
    try:
        initiator = Initiator(default_config_file, target_path)
        initiator.init()
    except Exception:
        # always in debug mode when init site
        logging.exception("Initialize site with error:")
        sys.exit(1)


def create_new_wiki(category, title, filename):
    if not filename:
        # `/` can't exists in filename
        _title = title.replace(os.sep, " slash ").lower()
        filename = "{0}.{1}".format(_title.replace(' ', '-'),
                                    config["default_ext"])
    now = datetime.datetime.now().strftime("%Y-%m-%d %H:%M")

    meta = "\n".join([
        "---",
        "title: \"{0}\"".format(title),
        "date: {0}".format(now),
        "---",
    ]) + "\n\n"

    category_path = os.path.join(config["source"], category)
    if not os.path.exists(category_path):
        mkdir_p(category_path)
        logger.info("Creating category: {0}.".format(category))

    fn = os.path.join(category_path, filename)
    if os.path.exists(fn):
        logger.warning("File exists: {0}".format(fn))
    else:
        logger.info("Creating wiki: {0}".format(fn))
        with io.open(fn, "wt", encoding="utf-8") as fd:
            fd.write(meta)


def preview_site(host, port, dest, root, do_watch):
    '''Preview site with watch content'''
    p_server = multiprocessing.Process(
        target=preview,
        args=(dest, root, host, port),
        name='ServerProcess'
    )
    p_server.start()

    if do_watch:
        base_path = getcwdu()
        p_watcher = multiprocessing.Process(
            target=watch,
            args=(config, base_path),
            name='WatcherProcess'
        )
        p_watcher.start()

    try:
        while p_server.is_alive():
            time.sleep(1)
        else:
            if do_watch:
                p_watcher.terminate()
    except (KeyboardInterrupt, SystemExit):
        # manually terminate process?
        pass


def update_builtin():
    '''Update builtin scripts and themes under local site'''
    # for fabfile.py
    yes_ans = ('y', 'yes')
    _fabfile_r = os.path.join(os.path.dirname(__file__), 'conf_templates',
                              'fabfile.py')
    _fabfile_l = os.path.join(os.getcwd(), 'fabfile.py')
    if os.path.exists(_fabfile_l):
        # py3 require md5 with bytes object, otherwise raise
        # TypeError: Unicode-objects must be encoded before hashing
        with open(_fabfile_r, 'rb') as _fd:
            _fabfile_r_md5 = hashlib.md5(_fd.read()).hexdigest()
        with open(_fabfile_l, 'rb') as _fd:
            _fabfile_l_md5 = hashlib.md5(_fd.read()).hexdigest()
        if _fabfile_l_md5 != _fabfile_r_md5:
            try:
                _ans = raw_input('Overwrite fabfile.py? (y/N) ')
                if _ans.lower() in yes_ans:
                    shutil.copy2(_fabfile_r, _fabfile_l)
            except (KeyboardInterrupt, SystemExit):
                print()  # newline with Ctrl-C
    else:
        try:
            _ans = raw_input('New fabfile.py? (y/N) ')
            if _ans.lower() in yes_ans:
                shutil.copy2(_fabfile_r, _fabfile_l)
        except (KeyboardInterrupt, SystemExit):
            print()

    # for themes
    _themes_r = os.path.join(os.path.dirname(__file__), 'themes')
    _themes_l = os.path.join(os.getcwd(), config['themes_dir'])
    for theme in os.listdir(_themes_r):
        _theme_r = os.path.join(_themes_r, theme)
        _theme_l = os.path.join(_themes_l, theme)
        if os.path.exists(_theme_l):
            _need_update = False
            for root, dirs, files in os.walk(_theme_r):
                files = [f for f in files if not f.startswith(".")]
                dirs[:] = [d for d in dirs if not d.startswith(".")]
                for filename in files:
                    with open(os.path.join(root, filename), 'rb') as _fd:
                        _theme_r_md5 = hashlib.md5(_fd.read()).hexdigest()
                    _dir = os.path.relpath(root, _theme_r)
                    with open(os.path.join(_theme_l, _dir, filename),
                              'rb') as _fd:
                        _theme_l_md5 = hashlib.md5(_fd.read()).hexdigest()
                    if _theme_l_md5 != _theme_r_md5:
                        _need_update = True
                        break
                if _need_update:
                    break
            if _need_update:
                try:
                    _ans = raw_input('Overwrite theme {0}? (y/N) '
                                     .format(theme))
                    if _ans.lower() in yes_ans:
                        shutil.rmtree(_theme_l)
                        copytree(_theme_r, _theme_l)
                except (KeyboardInterrupt, SystemExit):
                    print()
        else:
            try:
                _ans = raw_input('New theme {0}? (y/N) '.format(theme))
                if _ans.lower() in yes_ans:
                    copytree(_theme_r, _theme_l)
            except (KeyboardInterrupt, SystemExit):
                print()


def method_proxy(cls_instance, method_name, *args, **kwargs):
    '''ref: http://stackoverflow.com/a/10217089/1276501'''
    return getattr(cls_instance, method_name)(*args, **kwargs)


class Generator(object):

    def __init__(self, target_path):
        self.config = config
        self.config.update({'version': __version__})
        self.target_path = target_path
        self.pages = {}
        self.page_count = 0

    def generate(self):
        logger.debug("Empty the destination directory")
        dest_dir = os.path.join(self.target_path,
                                self.config["destination"])
        if os.path.exists(dest_dir):
            # for github pages
            exclude_list = ['.git', 'CNAME']
            emptytree(dest_dir, exclude_list)

        self.generate_pages()

        if not os.path.exists(os.path.join(self.config['source'], 'index.md')):
            self.generate_catalog(self.pages)

        feed_fn = 'atom.xml'
        if os.path.exists(os.path.join(getcwdu(), feed_fn)):
            self.generate_feed(self.pages, feed_fn)

        self.install_theme()

        self.copy_attach()

        # for github pages with custom domain
        cname_file = os.path.join(getcwdu(), 'CNAME')
        if os.path.exists(cname_file):
            shutil.copy2(cname_file,
                         os.path.join(self.config['destination'], 'CNAME'))

    def generate_feed(self, pages, feed_fn):
        logger.info("Generate feed.")
        feed_generator = FeedGenerator(self.config, self.target_path, pages,
                                       feed_fn)
        feed = feed_generator.generate_feed()
        ofile = os.path.join(
            self.target_path,
            self.config["destination"],
            feed_fn
        )
        write_file(ofile, feed)

    def generate_catalog(self, pages):
        logger.info("Generate catalog page.")
        catalog_generator = CatalogGenerator(self.config, self.target_path,
                                             pages)
        html = catalog_generator.generate_catalog_html()
        ofile = os.path.join(
            self.target_path,
            self.config["destination"],
            "index.html"
        )
        write_file(ofile, html)

    def generate_pages(self):
        logger.info("Start generating markdown files.")
        content_path = self.config["source"]
        _pages_l = []

        for root, dirs, files in os.walk(content_path):
            files = [f for f in files if not f.startswith(".")]
            dirs[:] = [d for d in dirs if not d.startswith(".")]
            for filename in files:
                if not filename.endswith(self.config["default_ext"]):
                    continue
                md_file = os.path.join(root, filename)
                _pages_l.append(md_file)

        npage = len(_pages_l)
        if npage:
            nproc = min(multiprocessing.cpu_count(), npage)

            split_pages = [[] for n in xrange(0, nproc)]
            random.shuffle(_pages_l)

            for i in xrange(npage):
                split_pages[i % nproc].append(_pages_l[i])

            pool = multiprocessing.Pool(processes=nproc)
            results = []
            for n in xrange(nproc):
                r = pool.apply_async(
                    method_proxy,
                    (self, 'generate_multiple_pages', split_pages[n]),
                    callback=self._generate_callback
                )
                results.append(r)

            pool.close()
            for r in results:
                r.get()

        generate_result = "{0} files generated.".format(self.page_count)
        # for failed pages
        _err_npage = npage - self.page_count
        if _err_npage:
            generate_result += " {0} files failed.".format(_err_npage)
        logger.info(generate_result)

    def generate_multiple_pages(self, md_files):
        _pages = {}
        _page_count = 0
        for _f in md_files:
            try:
                page_meta = self.generate_single_page(_f)
            except Exception:
                page_meta = None
                logger.exception('{0} failed to generate:'.format(_f))
            if page_meta:
                _pages[_f] = page_meta
                _page_count += 1
        return _pages, _page_count

    def generate_single_page(self, md_file):
        logger.debug("Generate: {0}".format(md_file))
        page_generator = PageGenerator(self.config, self.target_path,
                                       os.path.realpath(md_file))
        html = page_generator.to_html()

        # ignore draft
        if not html:
            return None

        category, filename = os.path.split(md_file)
        category = os.path.relpath(category, self.config['source'])
        output_file = os.path.join(
            self.target_path,
            self.config['destination'],
            category,
            '{0}.html'.format(os.path.splitext(filename)[0])
        )

        write_file(output_file, html)
        meta = page_generator.meta
        return meta

    def _generate_callback(self, result):
        _pages, _count = result
        self.pages.update(_pages)
        self.page_count += _count

    def install_theme(self):
        """Copy static directory under theme to destination directory"""
        src_theme = os.path.join(self.target_path, self.config["themes_dir"],
                                 self.config["theme"], "static")
        dest_theme = os.path.join(self.target_path, self.config["destination"],
                                  "static")
        if os.path.exists(dest_theme):
            shutil.rmtree(dest_theme)

        copytree(src_theme, dest_theme)
        logging.debug("Installing theme: {0}".format(self.config["theme"]))

    def copy_attach(self):
        """Copy attach directory under root path to destination directory"""
        src_p = os.path.join(self.target_path, self.config['attach'])
        dest_p = os.path.join(self.target_path, self.config["destination"],
                              self.config['attach'])
        if os.path.exists(src_p):
            copytree(src_p, dest_p)


def unicode_docopt(args):
    for k in args:
        if isinstance(args[k], basestring) and \
           not isinstance(args[k], unicode):
            args[k] = args[k].decode('utf-8')


def main(args=None):
    global config

    if not args:
        args = docopt(__doc__, version="Simiki {0}".format(__version__))
    unicode_docopt(args)

    logging_init(logging.DEBUG)

    target_path = args['-p'] if args['-p'] else getcwdu()

    if args["init"]:
        init_site(target_path)
    else:
        config_file = os.path.join(target_path, "_config.yml")
        try:
            config = parse_config(config_file)
        except (Exception, YAMLError):
            # always in debug mode when parse config
            logging.exception("Parse config with error:")
            sys.exit(1)
        level = logging.DEBUG if config["debug"] else logging.INFO
        logging_init(level)   # reload logger

        if args["generate"] or args["g"]:
            generator = Generator(target_path)
            generator.generate()
        elif args["new"] or args["n"]:
            create_new_wiki(args["-c"], args["-t"], args["-f"])
        elif args["preview"] or args["p"]:
            args['--port'] = int(args['--port'])
            preview_site(args['--host'], args['--port'], config['destination'],
                         config['root'], args['-w'])
        elif args["update"]:
            update_builtin()
        else:
            # docopt itself will display the help info.
            pass

    logger.info("Done.")


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, unicode_literals

import os
import os.path
import sys
import logging

try:
    import SimpleHTTPServer as http_server
except ImportError:
    # py3
    import http.server as http_server

try:
    import SocketServer as socket_server
except ImportError:
    # py3
    import socketserver as socket_server

try:
    import urllib2 as urllib_request
except ImportError:
    # py3
    import urllib.request as urllib_request

try:
    from os import getcwdu
except ImportError:
    # py3
    from os import getcwd as getcwdu

URL_ROOT = None
PUBLIC_DIRECTORY = None


class Reuse_TCPServer(socket_server.TCPServer):
    allow_reuse_address = True


class YARequestHandler(http_server.SimpleHTTPRequestHandler):

    def translate_path(self, path):
        if URL_ROOT and self.path.startswith(URL_ROOT):
            if self.path == URL_ROOT or self.path == URL_ROOT + '/':
                return PUBLIC_DIRECTORY + '/index.html'
            else:
                return PUBLIC_DIRECTORY + path[len(URL_ROOT):]
        else:
            return http_server.SimpleHTTPRequestHandler \
                .translate_path(self, path)

    def do_GET(self):
        # redirect url
        if URL_ROOT and not self.path.startswith(URL_ROOT):
            self.send_response(301)
            self.send_header('Location', URL_ROOT + self.path)
            self.end_headers()
        http_server.SimpleHTTPRequestHandler.do_GET(self)


def preview(path, url_root, host='localhost', port=8000):
    '''
    :param path: directory path relative to current path
    :param url_root: `root` setted in _config.yml
    '''
    global URL_ROOT, PUBLIC_DIRECTORY

    if not host:
        host = 'localhost'
    if not port:
        port = 8000

    if url_root.endswith('/'):
        url_root = url_root[:-1]

    URL_ROOT = urllib_request.quote(url_root.encode('utf-8'))
    PUBLIC_DIRECTORY = os.path.join(getcwdu(), path)

    if os.path.exists(path):
        os.chdir(path)
    else:
        logging.error("Path {} not exists".format(path))
    try:
        Handler = YARequestHandler
        httpd = Reuse_TCPServer((host, port), Handler)
    except (OSError, IOError) as e:
        logging.error("Could not listen on port {0}".format(port))
        sys.exit(getattr(e, 'exitcode', 1))

    logging.info("Serving at: http://{0}:{1}{2}/".format(host, port, url_root))
    logging.info("Serving running... (Press CTRL-C to quit)")
    try:
        httpd.serve_forever()
    except (KeyboardInterrupt, SystemExit):
        logging.info("Shutting down server")
        httpd.socket.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import os.path
import unittest
import shutil

from simiki.initiator import Initiator


class TestInitiator(unittest.TestCase):

    def setUp(self):
        BASE_DIR = os.path.join(os.path.dirname(__file__), '..')
        self.config_file = os.path.join(BASE_DIR, "simiki", "conf_templates",
                                        "_config.yml.in")
        self.target_path = os.path.join(BASE_DIR, "tests", "_build")
        if os.path.exists(self.target_path):
            shutil.rmtree(self.target_path)
        self.files = [
            "_config.yml",
            "fabfile.py",
            os.path.join("content", "intro", "gettingstarted.md"),
        ]
        self.dirs = [
            "content",
            "output",
            "themes",
            os.path.join("themes", "simple"),
        ]

    def test_target_exist(self):
        """ test Initiator target path exist
        """

        i = Initiator(self.config_file, self.target_path)
        i.init()

        for f in self.files:
            self.assertTrue(os.path.isfile(os.path.join(self.target_path, f)))

        for d in self.dirs:
            self.assertTrue(os.path.isdir(os.path.join(self.target_path, d)))

    def test_target_invalid(self):
        """ test Initiator target path invalid, raise OSError
        """

        target_error = "/foo/bar/why/not"
        i = Initiator(self.config_file, target_error)
        self.assertRaises(OSError, lambda: i.init())

    def tearDown(self):
        if os.path.exists(self.target_path):
            shutil.rmtree(self.target_path)

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import os
import os.path
import sys
import io
import logging
import datetime
from copy import deepcopy
from pprint import pprint
import yaml

import tzlocal


class ConfigFileNotFound(Exception):
    pass


def _set_default_config():
    config = {
        "url": "",
        "title": "",
        "keywords": "",
        "description": "",
        "author": "",
        "root": "/",
        "source": "content",
        "destination": "output",
        "attach": "attach",
        "themes_dir": "themes",
        "theme": "simple",
        "default_ext": "md",
        "pygments": True,
        "debug": False,
        "time": datetime.datetime.now(tzlocal.get_localzone()),
    }
    return config


def _post_process(config):
    for k, v in config.items():
        if v is None:
            config[k] = ""

    if config["url"].endswith("/"):
        config["url"] = config["url"][:-1]

    return config


def get_default_config():
    return _post_process(_set_default_config())


def parse_config(config_file):
    if not os.path.exists(config_file):
        raise ConfigFileNotFound("{0} not exists".format(config_file))

    default_config = _set_default_config()

    with io.open(config_file, "rt", encoding="utf-8") as fd:
        config = yaml.load(fd)

    default_config.update(config)
    config = _post_process(default_config)

    return config

if __name__ == "__main__":
    # pylint: disable=pointless-string-statement
    """
    Usage:
        python -m simiki.config : to test config template
        python -m simiki.config _config.yml : to test _config.yml file in \
                                                curren dir
    """
    if len(sys.argv) == 1:
        base_dir = os.path.dirname(__file__)
        _config_file = os.path.join(base_dir, "conf_templates",
                                    "_config.yml.in")
    elif len(sys.argv) == 2:
        base_dir = os.getcwd()
        _config_file = os.path.join(base_dir, sys.argv[1])
    else:
        logging.error("Use the template config file by default, "
                      "you can specify the config file to parse. \n"
                      "Usage: `python -m simiki.config [_config.yml]'")
        sys.exit(1)

    pprint(parse_config(_config_file))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, with_statement, unicode_literals

import os
import os.path
import shutil
import unittest
import io
from copy import deepcopy

from simiki import cli
from simiki.utils import copytree, emptytree

test_path = os.path.dirname(os.path.abspath(__file__))
base_path = os.path.dirname(test_path)

INIT_ARGS = {
    u'--help': False,
    u'--version': False,
    u'-c': None,
    u'-f': None,
    u'-p': None,
    u'-t': None,
    u'generate': False,
    u'g': False,
    u'init': False,
    u'new': False,
    u'n': False,
    u'preview': False,
    u'p': False
}


class TestCliInit(unittest.TestCase):
    def setUp(self):
        self.args = deepcopy(INIT_ARGS)
        self.target_path = "_build"

        if os.path.exists(self.target_path):
            shutil.rmtree(self.target_path)
        self.files = [
            "_config.yml",
            "fabfile.py",
            os.path.join("content", "intro", "gettingstarted.md"),
            os.path.join("themes", "simple", "page.html"),
            os.path.join("themes", "simple", "static", "css", "style.css")
        ]
        self.dirs = [
            "content",
            "output",
            "themes",
            os.path.join("themes", "simple"),
        ]

    def test_init(self):
        os.chdir(test_path)
        self.args.update({u'init': True, u'-p': self.target_path})
        cli.main(self.args)
        for f in self.files:
            self.assertTrue(os.path.isfile(os.path.join(self.target_path, f)))

        for d in self.dirs:
            self.assertTrue(os.path.isdir(os.path.join(self.target_path, d)))

    def tearDown(self):
        if os.path.exists(self.target_path):
            shutil.rmtree(self.target_path)


class TestCliGenerate(unittest.TestCase):
    def setUp(self):
        self.args = deepcopy(INIT_ARGS)
        self.wiki_path = os.path.join(test_path, "mywiki_for_cli")
        self.output_path = os.path.join(self.wiki_path, "output")

        if os.path.exists(self.output_path):
            emptytree(self.output_path)

        config_file_tpl = os.path.join(base_path, 'simiki',
                                       'conf_templates', '_config.yml.in')
        self.config_file_dst = os.path.join(self.wiki_path, '_config.yml')
        shutil.copyfile(config_file_tpl, self.config_file_dst)

        s_themes_path = os.path.join(base_path, 'simiki', 'themes')
        self.d_themes_path = os.path.join(self.wiki_path, 'themes')
        if os.path.exists(self.d_themes_path):
            shutil.rmtree(self.d_themes_path)
        copytree(s_themes_path, self.d_themes_path)

        self.drafts = [
            os.path.join(self.output_path, "intro", "my_draft.html")
        ]
        self.files = [
            os.path.join(self.output_path, "index.html"),
            os.path.join(self.output_path, "intro", "gettingstarted.html")
        ]
        self.dirs = [
            self.output_path,
            os.path.join(self.output_path, "intro"),
        ]
        self.attach = [
            os.path.join(self.output_path, 'attach', 'images', 'linux',
                         'opstools.png'),
        ]
        self.static = [
            os.path.join(self.output_path, "static", "css", "style.css"),
        ]
        os.chdir(self.wiki_path)

    def test_generate(self):
        self.args.update({u'generate': True})
        cli.main(self.args)
        for f in self.drafts:
            self.assertFalse(os.path.isfile(os.path.join(self.wiki_path, f)))

        for f in self.files:
            self.assertTrue(os.path.isfile(os.path.join(self.wiki_path, f)))

        for d in self.dirs:
            self.assertTrue(os.path.isdir(os.path.join(self.wiki_path, d)))

        for f in self.attach:
            self.assertTrue(os.path.isdir(os.path.join(self.wiki_path, d)))

        for f in self.static:
            self.assertTrue(os.path.isdir(os.path.join(self.wiki_path, d)))

    def tearDown(self):
        os.remove(self.config_file_dst)
        if os.path.exists(self.d_themes_path):
            shutil.rmtree(self.d_themes_path)
        if os.path.exists(self.output_path):
            emptytree(self.output_path)


class TestCliNewWiki(unittest.TestCase):
    def setUp(self):
        wiki_path = os.path.join(test_path, 'mywiki_for_others')
        config_file_tpl = os.path.join(base_path, 'simiki',
                                       'conf_templates', '_config.yml.in')
        self.config_file_dst = os.path.join(wiki_path, '_config.yml')

        shutil.copyfile(config_file_tpl, self.config_file_dst)

        self.args = deepcopy(INIT_ARGS)
        self.title = "hello/simiki"
        self.category = os.path.join('my目录', 'sub-category')
        self.source_path = os.path.join(wiki_path, "content")
        self.odir = os.path.join(wiki_path, "content", self.category)
        self.odir_root = os.path.dirname(self.odir)

        os.chdir(wiki_path)
        if os.path.exists(self.odir_root):
            shutil.rmtree(self.odir_root)

    def test_new_wiki_without_file(self):
        ofile = os.path.join(self.odir, "hello-slash-simiki.md")

        self.args.update({u'new': True, u'-t': self.title,
                          u'-c': self.category})
        cli.main(self.args)
        self.assertTrue(os.path.isfile(ofile))

        with io.open(ofile, "rt", encoding="utf-8") as fd:
            lines = fd.read().rstrip().splitlines()
            # Ignore date line
            lines[2] = u''
        expected_lines = [u'---', u'title: "hello/simiki"', u'', u'---']
        assert lines == expected_lines

    def tearDown(self):
        os.remove(self.config_file_dst)
        if os.path.exists(self.odir_root):
            shutil.rmtree(self.odir_root)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import logging
from logging import getLogger, Formatter, StreamHandler

from simiki import utils
from simiki.compat import is_linux, is_osx


class ANSIFormatter(Formatter):
    """Use ANSI escape sequences to colored log"""

    def format(self, record):
        try:
            msg = super(ANSIFormatter, self).format(record)
        except:
            # for python2.6
            # Formatter is old-style class in python2.6 and type is classobj
            # another trick: http://stackoverflow.com/a/18392639/1276501
            msg = Formatter.format(self, record)

        lvl2color = {
            "DEBUG": "blue",
            "INFO": "green",
            "WARNING": "yellow",
            "ERROR": "red",
            "CRITICAL": "bgred"
        }

        rln = record.levelname
        if rln in lvl2color:
            return "[{0}]: {1}".format(
                utils.color_msg(lvl2color[rln], rln),
                msg
            )
        else:
            return msg


class NonANSIFormatter(Formatter):
    '''Non ANSI color format'''

    def format(self, record):
        try:
            msg = super(NonANSIFormatter, self).format(record)
        except:
            # for python2.6
            # Formatter is old-style class in python2.6 and type is classobj
            # another trick: http://stackoverflow.com/a/18392639/1276501
            msg = Formatter.format(self, record)

        rln = record.levelname
        return "[{0}]: {1}".format(rln, msg)


def _is_platform_allowed_ansi():
    '''ansi be used on linux/macos'''
    if is_linux or is_osx:
        return True
    else:
        return False


def logging_init(level=None, logger=getLogger(),
                 handler=StreamHandler(), use_color=True):
    if use_color and _is_platform_allowed_ansi():
        fmt = ANSIFormatter()
    else:
        fmt = NonANSIFormatter()
    handler.setFormatter(fmt)
    logger.addHandler(handler)

    if level:
        logger.setLevel(level)


if __name__ == "__main__":
    logging_init(level=logging.DEBUG)

    root_logger = logging.getLogger()
    root_logger.debug("debug")
    root_logger.info("info")
    root_logger.warning("warning")
    root_logger.error("error")
    root_logger.critical("critical")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function, absolute_import, with_statement

import os
import sys
import ftplib
import getpass
from fabric.api import env, local, task, settings
from fabric.colors import blue, red
import fabric.contrib.project as project
from simiki import config

# XXX must run fab in root path of wiki
configs = config.parse_config('_config.yml')

env.colorize_errors = True
SUPPORTED_DEPLOY_TYPES = ('rsync', 'git', 'ftp')


def do_exit(msg):
    print(red(msg))
    print(blue('Exit!'))
    sys.exit()


def get_rsync_configs():
    if 'deploy' in configs:
        for item in configs['deploy']:
            if item['type'] == 'rsync':
                return item
    return None

# cannot put this block in deploy_rsync() for env.hosts
rsync_configs = get_rsync_configs()
if rsync_configs:
    env.user = rsync_configs.get('user', 'root')
    # Remote host and username
    if 'host' not in rsync_configs:
        do_exit('Warning: rsync host not set in _config.yml!')
    env.hosts = [rsync_configs['host'],]

    # Local output path
    env.local_output = os.path.join(
        os.path.abspath(os.path.dirname(__file__)),
        configs['destination'])

    # Remote path to deploy output
    if 'dir' not in rsync_configs:
        do_exit('Warning: rsync dir not set in _config.yml!')
    env.remote_output = rsync_configs['dir']

    # Other options
    env.port = rsync_configs.get('port')
    env.rsync_delete = rsync_configs.get('delete', False)


def deploy_rsync(deploy_configs):
    '''for rsync'''
    project.rsync_project(
        local_dir=env.local_output.rstrip("/")+"/",
        remote_dir=env.remote_output.rstrip("/")+"/",
        delete=env.rsync_delete
    )


def deploy_git(deploy_configs):
    '''for pages service of such as github/gitcafe ...'''
    with settings(warn_only=True):
        res = local('which -s ghp-import; echo $?', capture=True)
        if int(res.strip()):
            do_exit('Warning: ghp-import not installed! '
                    'run: `pip install ghp-import`')
    output_dir = configs['destination']
    remote = deploy_configs.get('remote', 'origin')
    branch = deploy_configs.get('branch', 'gh-pages')
    # commit gh-pages branch and push to remote
    _mesg = 'Update output documentation'
    local('ghp-import -p -m "{0}" -r {1} -b {2} {3}' \
          .format(_mesg, remote, branch, output_dir))


def deploy_ftp(deploy_configs):
    '''for ftp'''
    conn_kwargs = {'host': deploy_configs['host']}
    login_kwargs = {}
    if 'port' in deploy_configs:
        conn_kwargs.update({'port': deploy_configs['port']})
    if 'user' in deploy_configs:
        login_kwargs.update({'user': deploy_configs['user']})
    if 'password' in deploy_configs:
        passwd = deploy_configs['password']
        # when set password key with no value, get None by yaml
        if passwd is None:
            passwd = getpass.getpass('Input your ftp password: ')
        login_kwargs.update({'passwd': passwd})

    ftp_dir = deploy_configs.get('dir', '/')
    output_dir = configs['destination']

    ftp = ftplib.FTP()
    ftp.connect(**conn_kwargs)
    ftp.login(**login_kwargs)

    for root, dirs, files in os.walk(output_dir):
        rel_root = os.path.relpath(root, output_dir)
        for fn in files:
            store_fn = os.path.join(ftp_dir, rel_root, fn)
            ftp.storbinary('STOR %s' % store_fn,
                           open(os.path.join(root, fn), 'rb'))

    ftp.close()


@task
def deploy(type=None):
    '''deploy your site, support rsync / ftp / github pages

    run deploy:
        $ fab deploy

    run deploy with specific type(not supported specify multiple types):
        $ fab deploy:type=rsync

    '''
    if 'deploy' not in configs or not isinstance(configs['deploy'], list):
        do_exit('Warning: deploy not set right in _config.yml')
    if type and type not in SUPPORTED_DEPLOY_TYPES:
        do_exit('Warning: supported deploy type: {0}'
                .format(', '.join(SUPPORTED_DEPLOY_TYPES)))

    deploy_configs = configs['deploy']

    done = False

    for deploy_item in deploy_configs:
        deploy_type = deploy_item.pop('type')
        if type and deploy_type != type:
            continue
        func_name = 'deploy_{0}'.format(deploy_type)
        func = globals().get(func_name)
        if not func:
            do_exit('Warning: not supprt {0} deploy method'.format(deploy_type))
        func(deploy_item)
        done = True

    if not done:
        if type:
            do_exit('Warning: specific deploy type not configured yet')
        else:
            print(blue('do nothing...'))


@task
def commit():
    '''git commit source changes from all tracked/untracked files'''
    message = 'Update Documentation'
    commit_file = '-A'  # include tracked and untracked files

    with settings(warn_only=True):
        # Changes not staged for commit
        res = local('git status --porcelain 2>/dev/null | grep "^ M" | wc -l',
                    capture=True)
        if int(res.strip()):
            local("git add {0}".format(commit_file))

        # Changes to be committed
        res = local('git status --porcelain 2>/dev/null | grep "^M" | wc -l',
                    capture=True)
        if int(res.strip()):
            local("git commit -m '{0}'".format(message))



#!/usr/bin/env python
from __future__ import with_statement
import os
from setuptools import setup, find_packages
import simiki


entry_points = {
    "console_scripts": [
        "simiki = simiki.cli:main",
    ]
}

with open("requirements.txt") as f:
    requires = [l for l in f.read().splitlines() if l]

readme = "README.md"
if os.path.exists("README.rst"):
    readme = "README.rst"
with open(readme) as f:
    long_description = f.read()


setup(
    name="simiki",
    version=simiki.__version__,
    url="http://simiki.org/",
    author="Tanky Woo",
    author_email="me@tankywoo.com",
    description="Simiki is a simple wiki framework, written in Python.",
    long_description=long_description,
    keywords="simiki, wiki, generator",
    license="MIT License",
    packages=find_packages(),
    include_package_data=True,
    install_requires=requires,
    entry_points=entry_points,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS',
        'Operating System :: POSIX',
        'Operating System :: POSIX :: Linux',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    tests_require=['nose'],
    test_suite='nose.collector',
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals, absolute_import

import os
import os.path
import shutil
import errno
import logging
import io
import simiki
from simiki.compat import unicode

logger = logging.getLogger(__name__)

COLOR_CODES = {
    "reset": "\033[0m",
    "black": "\033[1;30m",
    "red": "\033[1;31m",
    "green": "\033[1;32m",
    "yellow": "\033[1;33m",
    "blue": "\033[1;34m",
    "magenta": "\033[1;35m",
    "cyan": "\033[1;36m",
    "white": "\033[1;37m",
    "bgred": "\033[1;41m",
    "bggrey": "\033[1;100m",
}


def color_msg(color, msg):
    return COLOR_CODES[color] + msg + COLOR_CODES["reset"]


def check_extension(filename):
    """Check if the file extension is in the allowed extensions

    The `fnmatch` module can also get the suffix:
        patterns = ["*.md", "*.mkd", "*.markdown"]
        fnmatch.filter(files, pattern)
    """
    exts = ['.{0}'.format(e) for e in simiki.allowed_extensions]
    return os.path.splitext(filename)[1] in exts


def copytree(src, dst, symlinks=False, ignore=None):
    """Copy from source directory to destination"""

    if not os.path.exists(dst):
        os.makedirs(dst)
    for item in os.listdir(src):
        s = os.path.join(src, item)
        d = os.path.join(dst, item)
        if os.path.isdir(s):
            copytree(s, d, symlinks, ignore)
        else:
            shutil.copy2(s, d)


def emptytree(directory, exclude_list=None):
    """Delete all the files and dirs under specified directory"""

    if not isinstance(directory, unicode):
        directory = unicode(directory, 'utf-8')
    if not exclude_list:
        exclude_list = []
    for p in os.listdir(directory):
        if p in exclude_list:
            continue
        fp = os.path.join(directory, p)
        if os.path.isdir(fp):
            try:
                shutil.rmtree(fp)
                logger.debug("Delete directory %s", fp)
            except OSError as e:
                logger.error("Unable to delete directory %s: %s",
                             fp, unicode(e))
        elif os.path.isfile(fp):
            try:
                logging.debug("Delete file %s", fp)
                os.remove(fp)
            except OSError as e:
                logger.error("Unable to delete file %s: %s", fp, unicode(e))
        else:
            logger.error("Unable to delete %s, unknown filetype", fp)


def mkdir_p(path):
    """Make parent directories as needed, like `mkdir -p`"""
    try:
        os.makedirs(path)
    except OSError as exc:  # Python >2.5
        # if dir exists, not error
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise


def listdir_nohidden(path):
    """List not hidden files or directories under path"""
    for f in os.listdir(path):
        if isinstance(f, str):
            f = unicode(f)
        if not f.startswith('.'):
            yield f


def write_file(filename, content):
    """Write content to file."""
    _dir, _ = os.path.split(filename)
    if not os.path.exists(_dir):
        logging.debug("The directory %s not exists, create it", _dir)
        mkdir_p(_dir)
    with io.open(filename, "wt", encoding="utf-8") as fd:
        fd.write(content)


if __name__ == "__main__":
    print(color_msg("black", "Black"))
    print(color_msg("red", "Red"))
    print(color_msg("green", "Green"))
    print(color_msg("yellow", "Yellow"))
    print(color_msg("blue", "Blue"))
    print(color_msg("magenta", "Magenta"))
    print(color_msg("cyan", "Cyan"))
    print(color_msg("white", "White"))
    print(color_msg("bgred", "Background Red"))
    print(color_msg("bggrey", "Background Grey"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import os.path
import unittest
import datetime
from simiki.config import parse_config, get_default_config

test_path = os.path.dirname(os.path.abspath(__file__))


class TestParseConfig(unittest.TestCase):
    def setUp(self):
        wiki_path = os.path.join(test_path, 'mywiki_for_others')

        self.expected_config = get_default_config()
        self.expected_config.update({
            "author": "Tanky Woo",
            "debug": True,
            "default_ext": "markdown",
            "description": "This is a simiki's config sample, \u6d4b\u8bd5\u6837\u4f8b",
            "destination": "destination",
            "keywords": "wiki, simiki, python, \u7ef4\u57fa",
            "root": "/wiki/",
            "source": "source",
            "attach": "attach",
            "theme": "mytheme",
            "themes_dir": "simiki_themes",
            "title": "\u6211\u7684Wiki",
            "url": "http://wiki.tankywoo.com"
        })
        self.config_file = os.path.join(wiki_path, "config_sample.yml")

    def test_parse_config(self):
        config = parse_config(self.config_file)
        self.expected_config.pop('time')
        _date = config.pop('time')
        if hasattr(unittest.TestCase, 'assertIsInstance'):
            self.assertIsInstance(_date, datetime.datetime)
        else:
            assert isinstance(_date, datetime.datetime), \
                   '%s is not an instance of %r' % \
                   (repr(_date), datetime.datetime)
        self.assertEqual(
            config,
            self.expected_config
        )

    def test_parse_config_not_exist(self):
        not_exist_config_file = os.path.join(self.config_file, "not_exist")
        self.assertRaises(Exception,
                          lambda: parse_config(not_exist_config_file))


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Python compat for python version and os system
'''
import sys

# Syntax sugar.
_ver = sys.version_info

#: Python 2.x?
is_py2 = (_ver[0] == 2)

#: Python 3.x?
is_py3 = (_ver[0] == 3)


_platform = sys.platform

# Windows
is_windows = _platform.startswith('win32')

# Linux
is_linux = _platform.startswith('linux')

# Mac OS X
is_osx = _platform.startswith('darwin')

# TODO Windows/Cygwin for 'cygwin'?


# Specifics

if is_py2:
    unicode = unicode
    basestring = basestring
    xrange = xrange
    raw_input = raw_input

if is_py3:
    unicode = str
    basestring = (str, bytes)
    xrange = range
    raw_input = input
